//
//	main.cpp - source file
//
//	David Henry - tfc_duke@club-internet.fr
//


#ifdef _WIN32
#define	WIN32_LEAN_AND_MEAN
#include	<windows.h>
// Microsoft Visual C++
#ifdef _MSC_VER
#pragma comment( lib, "opengl32.lib" )
#pragma comment( lib, "glu32.lib" )
#pragma comment( lib, "SDL.lib" )
#pragma comment( lib, "SDLmain.lib" )
#endif	// _MSC_VER
#endif	// WIN32

#include	<GL/gl.h>
#include	<GL/glu.h>
#include	<iostream>
#include	<iomanip>
#include	<SDL.h>

#include	"gl_ext.h"
#include	"texture.h"
#include	"md2.h"

using namespace std;


/////////////////////////////////////////////////
// modle
CMD2Model drfreak_md2;
CEntity drfreak;

int		iRenderMode = MD2E_DRAWNORMAL | MD2E_DRAWBUMP | MD2E_ANIMATED | MD2E_TEXTURED;

bool	bTextured	= true;
bool	bLightGL	= true;
bool	bAnimated	= true;

// surface SDL
SDL_Surface *surface;

// camra
struct vector3d
{
	union
	{
		struct { float	x, y, z; };
		float vec[3];
	};

} rot, eye;

vector3d g_kLightPosition = { 0.0, 2.0, 3.0 };
/////////////////////////////////////////////////



// ----------------------------------------------
// Shutdown() - fin du programme.
// ----------------------------------------------

void Shutdown( int returnCode )
{
	// destruction du texture manager
	CTextureManager::Kill();

	// clean up the window
	SDL_Quit();

	// quitte proprement
	exit( returnCode );
}



// ----------------------------------------------
// Initialize() - initialisation d'opengl.
// ----------------------------------------------

void Initialize( void )
{
	glClearColor( 0.5f, 0.5f, 0.5f, 0.0f );
	glShadeModel( GL_SMOOTH );

	glEnable( GL_DEPTH_TEST );
	glDepthFunc( GL_LEQUAL );

	glColor4f( 1.0, 1.0, 1.0, 1.0 );

	/////////////////////////////////////////////

	// initialisation camra
	rot.x = 0.0;		eye.x = 0.0;
	rot.y = 0.0;		eye.y = 0.0;
	rot.z = 0.0;		eye.z = 7.5;

	// initialisation des extensions OpenGL
	if( !InitExtensions() )
		Shutdown(1);

	// initialisation du texture manager
	CTextureManager *pTexMgr = CTextureManager::GetInstance();
	pTexMgr->Initialize();
	pTexMgr->MakeNormalizeVectorCubeMap( 64 );

	// chargement des modles
	drfreak_md2.LoadModel( "models/drfreak.md2" );
	drfreak_md2.LoadTexture( "models/drfreak.tga" );
	drfreak_md2.LoadNormalMap( "models/drfreak_bump.tga" );

	// initialisation des entits
	drfreak.SetModel( &drfreak_md2 );
	drfreak.SetScale( 0.1f );

	/////////////////////////////////////////////

	glEnable( GL_LIGHT0 );

	GLfloat lightpos[] = { 5.0f, 10.0f, 0.0f, 1.0f };
	GLfloat lightcolor[] = { 1.0f, 1.0f, 1.0f, 1.0f };

	glLightfv( GL_LIGHT0, GL_POSITION, lightpos );
	glLightfv( GL_LIGHT0, GL_AMBIENT, lightcolor );
	glLightfv( GL_LIGHT0, GL_DIFFUSE, lightcolor );
	glLightfv( GL_LIGHT0, GL_SPECULAR, lightcolor );
}



// ----------------------------------------------
// Resize() - rinitialisation de la fentre
// lors d'un redimensionnement...
// ----------------------------------------------

void Resize( GLint width, GLint height )
{
	// on vite les divisions par zro...
	if( height == 0 )
		height = 1;

	glViewport( 0, 0, width, height );

	// on rinitialise la matrice de projection
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0, (GLfloat)width/(GLfloat)height, 0.1, 1000.0 );

	// on rinitialise la matrice de modlisation/visualisation
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
}



// ----------------------------------------------
// Display() - redessine la scne opengl.
// ----------------------------------------------

void Display( void )
{
	// on nettoie l'cran pour redesiner la scne
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	glLoadIdentity();


	// rotation camra
	glTranslatef( -eye.x, -eye.y, -eye.z );
	glRotatef( rot.x, 1.0f, 0.0f, 0.0f );
	glRotatef( rot.y, 0.0f, 1.0f, 0.0f );
	glRotatef( rot.z, 0.0f, 0.0f, 1.0f );

	// position lumire
	glPointSize( 8.0 );
	glBegin( GL_POINTS );
		glVertex3fv( g_kLightPosition.vec );
	glEnd();


	// calcul fps
	static double curent_time = 0;
	static double last_time = 0;

	last_time = curent_time;
	curent_time = (double)SDL_GetTicks() / 1000.0;

	double fps = 1.0 / (curent_time - last_time);
//	cout << "   fps = " << setprecision(3) << fps << endl;


	if( bLightGL )
		glEnable( GL_LIGHTING );


	// dessin des entits

	if( iRenderMode & MD2E_ANIMATED )
	{
		// animation des modles
		drfreak.Animate( 0, 39, 10/fps );
	}

	// dessins des modles
	drfreak.DrawEntity( -1, iRenderMode | MD2E_USEGLCMDS, g_kLightPosition.vec );

	glDisable( GL_LIGHTING );
}



// ----------------------------------------------
// Keyboard() - capture clavier.
// ----------------------------------------------

void handleKeyPress( SDL_keysym *keysym )
{
	switch( keysym->sym )
	{
		case SDLK_a:
		{
			if( iRenderMode & MD2E_ANIMATED )
				iRenderMode &= ~MD2E_ANIMATED;
			else
				iRenderMode |= MD2E_ANIMATED;

			break;
		}

		case SDLK_b:
		{
			if( iRenderMode & MD2E_DRAWBUMP )
				iRenderMode &= ~MD2E_DRAWBUMP;
			else
				iRenderMode |= MD2E_DRAWBUMP;

			break;
		}

		case SDLK_c:
		{
			if( iRenderMode & MD2E_CULLBACKFACE )
				iRenderMode &= ~MD2E_CULLBACKFACE;
			else
				iRenderMode |= MD2E_CULLBACKFACE;

			break;
		}

		case SDLK_l:
		{
			bLightGL = !bLightGL;
			break;
		}

		case SDLK_s:
		{
			glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
			break;
		}

		case SDLK_t:
		{
			if( iRenderMode & MD2E_TEXTURED )
				iRenderMode &= ~MD2E_TEXTURED;
			else
				iRenderMode |= MD2E_TEXTURED;

			break;
		}

		case SDLK_w:
		{
			glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
			break;
		}

		case SDLK_RIGHT:
		{
			g_kLightPosition.x += 0.1f;
			break;
		}

		case SDLK_LEFT:
		{
			g_kLightPosition.x -= 0.1f;
			break;
		}

		case SDLK_UP:
		{
			g_kLightPosition.y += 0.1f;
			break;
		}

		case SDLK_DOWN:
		{
			g_kLightPosition.y -= 0.1f;
			break;
		}

		case SDLK_PAGEUP:
		{
			g_kLightPosition.z += 0.1f;
			break;
		}

		case SDLK_PAGEDOWN:
		{
			g_kLightPosition.z -= 0.1f;
			break;
		}

		case SDLK_F1:
		{
			SDL_WM_ToggleFullScreen( surface );
			break;
		}

		case SDLK_ESCAPE:
		{
			Shutdown(0);
			break;
		}

		default:
			break;
	}
}



// ----------------------------------------------
// MouseMove() - contrle souris.
// ----------------------------------------------

void MouseMove( int x, int y )
{
	static int x_pos = 0;
	static int y_pos = 0;

	SDLMod km = SDL_GetModState();

	if( SDL_GetMouseState( NULL, NULL ) & SDL_BUTTON_RMASK ) // right
	{
		// zoom
		eye.z += (x - x_pos) * 0.1;
	}
	else if( SDL_GetMouseState( NULL, NULL ) & SDL_BUTTON_LMASK ) // left
	{
		if( (km & KMOD_RSHIFT) || (km & KMOD_LSHIFT))
		{
			// translation
			eye.x -= (x - x_pos) * 0.1;
			eye.y += (y - y_pos) * 0.1;
		}
		else
		{
			// rotation
			rot.x += (y - y_pos);
			rot.y += (x - x_pos);
		}
	}

	x_pos = x;
	y_pos = y;
}



// ----------------------------------------------
// main() - fonction principale.
// ----------------------------------------------

int main( int argc, char *argv[] )
{
	cout << "OpenGL MD2 Bump Mapping demo // by David Henry : tfc_duke@club-internet.fr" << endl;
	cout << "Built release " << __DATE__ << " at " << __TIME__ << endl;

	int videoFlags;
	bool done = false;
	SDL_Event event;
	const SDL_VideoInfo *videoInfo;
	bool isActive = true;


	// initialisation de SDL
	if( SDL_Init( SDL_INIT_VIDEO ) < 0 )
	{
		cout << "Video initialization failed: " << SDL_GetError() << endl;
		Shutdown(1);
	}

	// on rcupre les infos video
	videoInfo = SDL_GetVideoInfo();

	if( !videoInfo )
	{
		cout << "Video query failed: " << SDL_GetError() << endl;
		Shutdown(1);
	}

	// flags  passer  SDL_SetVideoMode
	videoFlags  = SDL_OPENGL;			// utilise OpenGL avec SDL
	videoFlags |= SDL_GL_DOUBLEBUFFER;	// utilise le double buffering
	videoFlags |= SDL_HWPALETTE;		// pas d'approximations pour les couleurs en mode 8 bits
	videoFlags |= SDL_RESIZABLE;		// accepte le redimensionnement de la fentre

	// check si la surface peut-tre alloue en mmoire video
	if( videoInfo->hw_available )
		videoFlags |= SDL_HWSURFACE;
	else
		videoFlags |= SDL_SWSURFACE;

    // check si hardware blits possible
	if( videoInfo->blit_hw )
		videoFlags |= SDL_HWACCEL;

	// initialisation du double buffering pour OpenGL
	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );

	// initialise un mode video
	surface = SDL_SetVideoMode( 640, 480, 24, videoFlags );
	SDL_WM_SetCaption( "MD2 and DOT3 Bump Mapping demo", "MD2 and DOT3 Bump Mapping demo" );


	// vrifie que la surface n'est pas nulle
	if( !surface )
	{
		cout << "Video mode set failed: " << SDL_GetError() << endl;
		Shutdown(1);
	}

	// active la rptition des touches clavier
	if( (SDL_EnableKeyRepeat( 100, 100 /*SDL_DEFAULT_REPEAT_INTERVAL*/ ) ) )
	{
		cout << "Setting keyboard repeat failed: " << SDL_GetError() << endl;
		Shutdown(1);
	}

	// initialise OpenGL
	Initialize();

	// redimensionne la fentre initiale
	Resize( 640, 480 );

	// boucle principale (jusqu' la fin du programme)
	while( !done )
	{
		// traite les vnements SDL
		while( SDL_PollEvent( &event ) )
		{
			switch( event.type )
			{
				case SDL_ACTIVEEVENT:
				{
					// le focus de la fentre a t altr (fentre iconifie ou maximise)
					// on ne dessine plus la scne OpenGL
					if( (event.active.gain == 0) && (event.active.state & SDL_APPACTIVE) )
						isActive = false;
					else
						isActive = true;

					break;
				}

				case SDL_VIDEORESIZE:
				{
					// redimensionnement de la fentre
					surface = SDL_SetVideoMode( event.resize.w, event.resize.h, 24, videoFlags );

					if( !surface )
					{
						cout << "Could not get a surface after resize: " << SDL_GetError() << endl;
						Shutdown( 1 );
					}

					Resize( event.resize.w, event.resize.h );
					break;
				}

				case SDL_KEYDOWN:
				{
					// une touche a t presse
					handleKeyPress( &event.key.keysym );
					break;
				}

				case SDL_MOUSEMOTION:
				{
					// mouvement de la souris
					MouseMove( event.button.x, event.button.y );
					break;
				}

				case SDL_QUIT:
				{
					// fin du programme
					done = true;
					break;
				}

				default:
					break;
			}
		}

		// si la fentre est active
		if( isActive )
		{
			// dessine la scne
			Display();

			// change les tampons avant et arrire
			SDL_GL_SwapBuffers();
		}
	}

	// dtruit les objets et quitte le programme
	Shutdown(0);

	// ne devrait jamais tre excut (cf: Shutdown() appelle exit())
	return 0;
}
