//
//	texture.h - header file
//
//	David Henry - tfc_duke@club-internet.fr
//


#ifndef		__TEXTURE_H
#define		__TEXTURE_H


#include	<map>
#include	<string>

#include	"singleton.h"



// ==============================================
// CTextureManager - OpenGL texture manager.
// ==============================================

class CTextureManager : public CSingleton<CTextureManager>
{
	friend class CSingleton<CTextureManager>;

private:
	// constructeur/destructeur
	CTextureManager( void ) { m_kDataBase.clear(); }
	~CTextureManager( void ) { ReleaseTextures(); }


public:
	// functions publiques
	void	Initialize( void );
	void	MakeNormalizeVectorCubeMap( int iSize );

	GLuint	LoadTexture( std::string szFilename, bool bFlip = false );

	GLuint	GetTexture( std::string szFilename );
	char	*GetTextureName( GLuint id );

	GLuint	GetDefaultTextureID( void )		{ return *m_pDefaultTextureID; }
	GLuint	GetNormCubeMapID( void )		{ return *m_pNormCubeMapID; }

	void	ReleaseTextures( void );

	void	DeleteTexture( std::string szName );
	void	DeleteTexture( GLuint id );


private:
	// fonctions prives
	void	GetCubeVector( int i, int cubesize, int x, int y, float *vector );


private:
	// texture map<>
	std::map<std::string, GLuint>::iterator	m_kItor;
	std::map<std::string, GLuint>			m_kDataBase;

	GLuint	*m_pDefaultTextureID;
	GLuint	*m_pNormCubeMapID;

};


#endif	// __TEXTURE_H
